@@ -26,16 +26,16 @@ from astropy.coordinates import SkyCoord
 from astropy.coordinates import Angle
 from astropy import units as u
 from astropy.time import Time
+from astroquery.sdss import SDSS
 
 from fink_science.conversion import dc_mag
 
 
 @pandas_udf(BooleanType(), PandasUDFType.SCALAR)
 def early_kn_candidates(
-        objectId, drb, classtar, jd,
-        jdstarthist, ndethist, cdsxmatch, fid,
-        magpsf, sigmapsf, magnr, sigmagnr,
-        magzpsci, isdiffpos, ra, dec, roid, field) -> pd.Series:
+        objectId, drb, classtar, jd, jdstarthist, ndethist, cdsxmatch, fid,
+        magpsf, sigmapsf, magnr, sigmagnr, magzpsci, isdiffpos, ra, dec, roid,
+        field) -> pd.Series:
     """
     Return alerts considered as KN candidates.
 
@@ -204,6 +204,29 @@ def early_kn_candidates(
 
         f_kn.loc[f_kn] = np.array(galaxy_matching, dtype=bool)
 
+    # check the nature of close objects in SDSS catalog
+    if f_kn.any():
+        no_star = []
+        for i in range(sum(f_kn)):
+            pos = SkyCoord(
+                ra=np.array(ra[f_kn])[i] * u.degree,
+                dec=np.array(dec[f_kn])[i] * u.degree
+                )
+            # for a test on "many" objects, you may wait 1s to stay under the
+            # query limit.
+            table = SDSS.query_region(pos, fields=['type'],
+                                      radius=5 * u.arcsec)
+            type_close_objects = []
+            if table is not None:
+                type_close_objects = table['type']
+            # types: 0: UNKNOWN, 1: STAR, 2: GALAXY, 3: QSO, 4: HIZ_QSO,
+            # 5: SKY, 6: STAR_LATE, 7: GAL_EM
+            to_remove_types = [1, 3, 4, 6]
+            no_star.append(
+                len(np.intersect1d(type_close_objects, to_remove_types)) == 0
+                )
+        f_kn.loc[f_kn] = np.array(no_star, dtype=bool)
+
     if f_kn.any():
         # Simplify notations
         b = gal.b.degree[f_kn]
@@ -244,9 +267,9 @@ def early_kn_candidates(
             *Measurement (band {}):*\n- Apparent magnitude: {:.2f} ± {:.2f}
             """.format(dict_filt[fid[i]], mag[i], err_mag[i])
         host_text = """
-            *Presumed host galaxy:*\n- Index in Mangrove catalog: {}\n- 2MASS XSC Name: {:s}\n- Luminosity distance: ({:.2f} ± {:.2f}) Mpc\n- RA/Dec: {:.7f} {:+.7f}\n- log10(Stellar mass/Ms): {:.2f}
+            *Presumed host galaxy:*\n- HyperLEDA Name: {:s}\n- 2MASS XSC Name: {:s}\n- Luminosity distance: ({:.2f} ± {:.2f}) Mpc\n- RA/Dec: {:.7f} {:+.7f}\n- log10(Stellar mass/Ms): {:.2f}
             """.format(
-            pdf_mangrove.loc[host_galaxies[i], 'galaxy_idx'],
+            pdf_mangrove.loc[host_galaxies[i], 'HyperLEDA_name'][2:-1],
             pdf_mangrove.loc[host_galaxies[i], '2MASS_name'][2:-1],
             pdf_mangrove.loc[host_galaxies[i], 'lum_dist'],
             pdf_mangrove.loc[host_galaxies[i], 'dist_err'],
@@ -266,6 +289,7 @@ def early_kn_candidates(
         """.format(ra_formatted[i], dec_formatted[i], ra[i], dec[i])
         galactic_position_text = """
             *Galactic latitude:*\n- [deg]: {:.7f}""".format(b[i])
+        tns_text = '*TNS:* <https://www.wis-tns.org/search?ra={}&decl={}&radius=5&coords_unit=arcsec|link>'.format(ra[i], dec[i])
         # message formatting
         blocks = [
             {
@@ -303,7 +327,11 @@ def early_kn_candidates(
                     {
                         "type": "mrkdwn",
                         "text": measurements_text
-                    }
+                    },
+                    {
+                        "type": "mrkdwn",
+                        "text": tns_text
+                    },
                 ]
             },
         ]
@@ -342,7 +370,7 @@ def early_kn_candidates(
                 os.environ['KNWEBHOOK_AMA_GALAXIES'],
                 json={
                     'blocks': blocks,
-                    'username': 'kilonova bot'
+                    'username': 'Cross-match-based kilonova bot'
                 },
                 headers={'Content-Type': 'application/json'},
             )
